<?php
// $Id: nutch.admin.inc,v 1.1 2010/04/11 23:40:26 dstuart Exp $

/**
 * @file
 * Implements admin functionality relating to Nutch setup and running
 */


/**
 * Settings form for Nutch setup
 * @TODO Need to add all of the extra crawl options 
 *        - topN: -topN, then instead of all unfetched urls you only get N urls with the highest score
 *        - threads
 *        - depth
 *        - adddays
 */
function nutch_admin_settings() {

  $form = array();
  $form['system'] = array(
    '#type' => 'fieldset',
    '#title' => t('System settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['system']['nutch_nutch_dir'] = array(
    '#type' => 'textfield',
    '#title' => t('NUTCH_HOME'),
    '#description' => t('The absolute path to where the Nutch distribution is found, so that $NUTCH_HOME/bin/nutch resolves to the main Nutch script.'),
    '#default_value' => variable_get('nutch_nutch_dir', '/usr/local/nutch'),
    '#required' => TRUE,
  );
  $form['system']['nutch_java'] = array(
    '#type' => 'textfield',
    '#title' => t('JAVA_HOME'),
    '#description' => t('The absolute path to a directory which is JAVA_HOME. $JAVA_HOME/bin/java should resolve to the Java binary that will run Nutch.'),
    '#default_value' => variable_get('nutch_java', '/usr'),
    '#required' => TRUE,
  );

  if (!$nutch_solr  = variable_get('nutch_solr', FALSE)) {
    $host = variable_get('apachesolr_host', 'localhost');
    $port = variable_get('apachesolr_port', '8983');
    $path = variable_get('apachesolr_path', '/solr');
    $nutch_solr = sprintf('http://%s:%s%s',$host, $port, $path);
  }
  $form['system']['nutch_solr'] = array(
    '#type' => 'textfield',
    '#title' => t('SOLR URL'),
    '#description' => t('The url of your apache solr instance.'),
    '#default_value' => $nutch_solr,
    '#required' => TRUE,
  );
  $form['system']['nutch_urls_to_fetch'] = array(
    '#type' => 'select',
    '#title' => t('The number of URLs to fetch and index'),
    '#options' => drupal_map_assoc(array(10, 20, 50, 100, 200, 500, 750, 1000, 2000, 5000, 10000)),
    '#default_value' => variable_get('nutch_urls_to_fetch', 100),
    '#description' => t('When crawl is run, this number of URLs will be fetched and indexed. It is important to know how long this takes on your equipment so as to avoid starting overlapping jobs. Always start with small numbers (hundreds) and profile how long it takes before moving up.'),
  );
  
  $form['system']['nutch_commit'] = array(
    '#type' => 'checkbox',
    '#title' => t('Send to Solr'),
    '#description' => t('Send to Solr at the end of the Nutch crawl'),
    '#default_value' => variable_get('nutch_commit', 1),
  );    
  $form['system']['nutch_debug'] = array(
    '#type' => 'checkbox',
    '#title' => t('Dry run/debug crawl'),
    '#default_value' => variable_get('nutch_debug', 0),
  );    

  $form['system']['nutch_crawl_on_cron'] = array(
    '#type' => 'checkbox',
    '#title' => t('Run crawl on cron'),
    '#default_value' => variable_get('nutch_crawl_on_cron', 0),
  );


  return system_settings_form($form);
}

function nutch_admin_crawl() {
  $form = array();
  $form['controls'] = array(
    '#type' => 'fieldset',
    '#title' => t('crawl controls'),
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
  );
  $form['controls']['nutch_commit'] = array(
    '#type' => 'checkbox',
    '#title' => t('Send to Solr'),
    '#description' => t('Send to Solr at the end of the Nutch crawl'),
    '#default_value' => variable_get('nutch_commit', 1),
  );    
  $form['controls']['nutch_debug'] = array(
    '#type' => 'checkbox',
    '#title' => t('Dry run/debug crawl'),
    '#default_value' => variable_get('nutch_debug', 0),
  );  
  $form['controls']['crawl'] = array(
    '#type' => 'submit',
    '#value' => t('Start Crawl'),
  );
  return $form;
}

function nutch_admin_crawl_submit($form, &$form_state) {
  nutch_start_crawl(
           variable_get('nutch_nutch_dir', '/usr/local/nutch'), 
           variable_get('nutch_java', '/usr'), 
           variable_get('nutch_solr', FALSE), 
           (@$form['#post']['nutch_commit']?$form['#post']['nutch_commit']:variable_get('nutch_commit', 1)), 
           variable_get('nutch_urls_to_fetch', 100), 
           variable_get('nutch_seed_url', 'http://localhost'), 
           (@$form['#post']['nutch_debug']?$form['#post']['nutch_debug']:variable_get('nutch_debug', 0))
  );
}

function nutch_start_crawl($nutch_home, $java_home, $solr_url, $commit=0, $fetch_amount=100, $seed_urls='', $debug=0) {
  
  $command =  $_SERVER['DOCUMENT_ROOT'] . base_path() . drupal_get_path('module', 'nutch') .'/'.'runbot';

  if (!empty($nutch_home)){
    $command .= ' -n '. escapeshellarg($nutch_home);
  }
  else {
    drupal_set_message(t("You must supply a nutch home directory; crawl aborted."));
    return 0;
  }
  
  if (!empty($java_home)) {
    $command .= ' -j '. escapeshellarg($java_home);
  }
  else {
    drupal_set_message(t("You must supply a java home; crawl aborted."));
    return 0;
  }  

  if (!empty($solr_url)) {
    $command .= ' -s '. escapeshellarg($solr_url);
  }
  else {
    drupal_set_message(t("You must the url path to your Solr instance; crawl aborted."));
    return 0;
  }

  if (!empty($seed_urls)) {
    /* 
     * Replace all of the line feeds with ! because the
     * shell scipt doesnt recognise them when adding them
     * to the urls file. We could run echo -e but its not
     * a universal command 
     */
    $seed_urls_replaced = str_replace("\n", '!', $seed_urls);
    if($debug == 1) drupal_set_message($seed_urls_replaced);
    $command .= ' -u '. escapeshellarg($seed_urls_replaced);
  }
  
  $command .= ' -c '. escapeshellarg($commit);
  $command .= ' -f '. escapeshellarg($fetch_amount);
  $command .= ' -d '. escapeshellarg($debug);
  
  drupal_set_message(t("Starting Nutch Crawl."));
  if($debug == 1){
    drupal_set_message($command);
    $rtn = exec($command . ' &', $output);
    foreach($output as $line){
      $rtn_output .= $line . "<br/>";
    }
    drupal_set_message($rtn_output);     
  }
  else{
    $tmp = array();
    $process = proc_open($command . ' &', array(), $tmp);
    drupal_set_message($process);
    $rtn = proc_close($process);
  }

}

function nutch_admin_seed() {
  $form = array();
  $form['controls'] = array(
    '#type' => 'fieldset',
    '#title' => t('seed controls'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  /* @TODO DO a compare of the current seed files on disk and that saved in the variables table
             if different set a message */ 
  $form['controls']['nutch_seed_url'] = array(
    '#type' => 'textarea',
    '#title' => t('Seed URLs'),
    '#description' => t('The URL/s Nutch uses seed crawl. Please urls on a separate line'),
    '#default_value' => variable_get('nutch_seed_url', 'http://localhost'),
    '#required' => TRUE,
  );  
  return system_settings_form($form);
}


function nutch_admin_inject_submit($form, &$form_state) {
    drupal_set_message(t("Nutch is injecting URLs."));
    nutch_do_inject(variable_get('nutch_nutch_dir', '/usr/local/nutch'), variable_get('nutch_crawl_dir', '/usr/local/nutch/crawl'), variable_get('nutch_urls_dir', '/usr/local/nutch/seed'));
}


function nutch_admin_logs() {
  $log_file = variable_get('nutch_nutch_dir', '/usr/local/nutch') .'/logs/hadoop.log';
  $output = check_plain(file_get_contents($log_file));
  if (!$output) {
    $output = t("The file $log_file couldn't be found.");
  }

  $form = array(
    'log' => array(
      '#type' => 'textarea',
      '#title' => $log_file,
      '#value' => $output,
      '#rows' => 30,
    ),
  );
  return $form;
}

function nutch_get_current_seed_urls(){
  $seed_file = variable_get('nutch_nutch_dir', '/usr/local/nutch') .'/seed/urls';
  $output = check_plain(file_get_contents($seed_file));
  if (!$output) {
    $output = '';
  }
  return $output;
}